/*
 * Copyright 2002 Draagon Software LLC. All Rights Reserved.
 *
 * This software is the proprietary information of Draagon Software LLC.
 * Use is subject to license terms.
 */

package com.draagon.meta.attr;

import com.draagon.meta.*;


/**
 * An attribute of a MetaClass, MetaField, or MetaView
 */
@SuppressWarnings("serial")
public class ClassAttribute extends MetaAttribute
{
  //private static Log log = LogFactory.getLog( ClassAttribute.class );

  private Class<?> mClass = null;

  /**
   * Constructs the MetaClass
   */
  public ClassAttribute()
  {
  }

  /**
   * Sets the value of the MetaAttribute
   */
  public void setValue( Object value )
    throws InvalidAttributeValueException
  {
    if ( value instanceof Class )
      mClass = (Class<?>) value;
    else if ( value instanceof String ) {
      String classStr = (String) value;
      try {
        mClass = Class.forName( classStr );
      }
      catch( ClassNotFoundException e ) {
        throw new InvalidAttributeValueException( "Invalid Class Name [" + classStr + "]" );
      }
    }
    else
      throw new InvalidAttributeValueException( "Unsupported Value Object [" + value + "], Class or String expected" );
  }

  /**
   * Returns the value of the MetaAttribute
   */
  public Object getValue()
  {
    return mClass;
  }

  public Object clone()
  {
    ClassAttribute ca = (ClassAttribute) super.clone();
    ca.mClass = mClass;
    return ca;
  }
}
